@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2023.12.17 22:00:00                  #
+# Updated Date: 2023.12.18 22:00:00                  #
 # ================================================== #
 
 import datetime
@@ -24,8 +24,13 @@ class Config:
     TYPE_FLOAT = 2
     TYPE_BOOL = 3
 
-    def __init__(self):
-        """Config handler"""
+    def __init__(self, window=None):
+        """
+        Config handler
+
+        :param window: Window instance
+        """
+        self.window = window
         self.path = str(Path(os.path.join(Path.home(), '.config', self.CONFIG_DIR)))
         self.initialized = False
         self.models = {}
@@ -46,21 +51,38 @@ class Config:
         else:
             return os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
 
+    def get_user_path(self):
+        """
+        Return user path
+
+        :return: user path
+        :rtype: str
+        """
+        return self.path
+
     def get_available_langs(self):
         """
         Return list with available languages
 
-        :return: list with available languages
+        :return: list with available languages (user + app)
         :rtype: list
         """
         langs = []
         path = os.path.join(self.get_root_path(), 'data', 'locale')
-        if not os.path.exists(path):
-            return langs
+        if os.path.exists(path):
+            for file in os.listdir(path):
+                if file.startswith('locale.') and file.endswith(".ini"):
+                    lang_id = file.replace('locale.', '').replace('.ini', '')
+                    if lang_id not in langs:
+                        langs.append(lang_id)
 
-        for file in os.listdir(path):
-            if file.endswith(".ini"):
-                langs.append(file.replace('locale.', '').replace('.ini', ''))
+        path = os.path.join(self.get_user_path(), 'locale')
+        if os.path.exists(path):
+            for file in os.listdir(path):
+                if file.startswith('locale.') and file.endswith(".ini"):
+                    lang_id = file.replace('locale.', '').replace('.ini', '')
+                    if lang_id not in langs:
+                        langs.append(lang_id)
 
         # make English first
         if 'en' in langs:
@@ -73,15 +95,18 @@ class Config:
         Initialize config
 
         :param all: load all configs
-        :param log: log loading
+        :param log: log loading to console
         """
         if not self.initialized:
             if all:
                 v = self.get_version()
-                print("PyGPT v{}".format(v))
+                os = self.window.platform.get_os()
+                architecture = self.window.platform.get_architecture()
+                print("")
+                print("PyGPT v{} ({}, {})".format(v, os, architecture))
                 print("Author: Marcin Szczyglinski")
-                print("Project website: https://pygpt.net")
                 print("GitHub: https://github.com/szczyglis-dev/py-gpt")
+                print("WWW: https://pygpt.net")
                 print("Email: info@pygpt.net")
                 print("")
                 print("Initializing...")
@@ -171,7 +196,7 @@ class Config:
             self.data = json.load(f)
             self.data = dict(sorted(self.data.items(), key=lambda item: item[0]))  # sort by key
             f.close()
-            if log:
+            if log:  # TODO: move to self
                 print("Loaded config: {}".format(path))
         except Exception as e:
             print(e)
@@ -670,7 +695,7 @@ class Config:
                 src = os.path.join(self.get_root_path(), 'data', 'config', 'models.json')
                 shutil.copyfile(src, dst)
 
-            # install prompt presets
+            # install presets
             presets_dir = os.path.join(self.path, 'presets')
             if not os.path.exists(presets_dir):
                 src = os.path.join(self.get_root_path(), 'data', 'config', 'presets')
@@ -710,4 +735,4 @@ class Config:
                 os.mkdir(capture_dir)
 
         except Exception as e:
-            print(e)
+            print("Error installing config files:", e)
