@@ -6,13 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2023.12.17 22:00:00                  #
+# Updated Date: 2023.12.18 23:00:00                  #
 # ================================================== #
 
 from PySide6.QtGui import QAction
 
 from ..dispatcher import Event
-from ..plugins import Plugins as Handler
 from ..utils import trans
 
 
@@ -24,7 +23,6 @@ class Plugins:
         :param window: Window instance
         """
         self.window = window
-        self.handler = Handler(self.window.config)
         self.config_dialog = False
         self.config_initialized = False
         self.current_plugin = None
@@ -43,8 +41,8 @@ class Plugins:
         """
         Set up plugins ui
         """
-        for id in self.handler.plugins:
-            plugin = self.handler.plugins[id]
+        for id in self.window.app.plugins.plugins:
+            plugin = self.window.app.plugins.plugins[id]
             try:
                 plugin.setup_ui()  # setup UI
             except AttributeError:
@@ -52,37 +50,44 @@ class Plugins:
         # show/hide UI elements
         self.handle_enabled_types()
 
+        # tmp dump locales
+        # self.window.app.plugins.dump_plugin_locales()
+
     def setup_settings(self):
         """Set up plugins settings"""
         idx = None
         # restore previous selected or restored tab on dialog create
-        if 'plugin.settings' in self.window.tabs:
-            idx = self.window.tabs['plugin.settings'].currentIndex()
+        if 'plugin.settings' in self.window.ui.tabs:
+            idx = self.window.ui.tabs['plugin.settings'].currentIndex()
         self.window.plugin_settings.setup(idx)  # widget dialog Plugins
 
     def setup_menu(self):
         """Set up plugins menu"""
-        for id in self.handler.plugins:
-            if id in self.window.menu['plugins']:
+        for id in self.window.app.plugins.plugins:
+            plugin = self.window.app.plugins.plugins[id]
+            if id in self.window.ui.menu['plugins']:
                 continue
-            default_name = self.handler.plugins[id].name
+            default_name = self.window.app.plugins.plugins[id].name
             trans_key = 'plugin.' + id
             name = trans(trans_key)
             if name == trans_key:
                 name = default_name
-            self.window.menu['plugins'][id] = QAction(name, self.window, checkable=True)
-            self.window.menu['plugins'][id].triggered.connect(
+            if plugin.use_locale:
+                domain = 'plugin.{}'.format(id)
+                name = trans('plugin.name', False, domain)
+            self.window.ui.menu['plugins'][id] = QAction(name, self.window, checkable=True)
+            self.window.ui.menu['plugins'][id].triggered.connect(
                 lambda checked=None, id=id: self.window.controller.plugins.toggle(id))
-            self.window.menu['menu.plugins'].addAction(self.window.menu['plugins'][id])
+            self.window.ui.menu['menu.plugins'].addAction(self.window.ui.menu['plugins'][id])
 
     def update(self):
         """Update plugins menu"""
-        for id in self.window.menu['plugins']:
-            self.window.menu['plugins'][id].setChecked(False)
+        for id in self.window.ui.menu['plugins']:
+            self.window.ui.menu['plugins'][id].setChecked(False)
 
         for id in self.enabled:
             if self.enabled[id]:
-                self.window.menu['plugins'][id].setChecked(True)
+                self.window.ui.menu['plugins'][id].setChecked(True)
 
         self.handle_enabled_types()
 
@@ -90,8 +95,8 @@ class Plugins:
         """
         Destroy plugins workers
         """
-        for id in self.handler.plugins:
-            plugin = self.handler.plugins[id]
+        for id in self.window.app.plugins.plugins:
+            plugin = self.window.app.plugins.plugins[id]
             try:
                 plugin.destroy()  # destroy plugin workers
             except AttributeError:
@@ -120,12 +125,12 @@ class Plugins:
 
         # select first plugin on list if no plugin selected yet
         if selected_plugin is None:
-            if len(self.handler.plugins) > 0:
-                selected_plugin = list(self.handler.plugins.keys())[0]
+            if len(self.window.app.plugins.plugins) > 0:
+                selected_plugin = list(self.window.app.plugins.plugins.keys())[0]
 
         # assign plugin options to config dialog fields
-        for id in self.handler.plugins:
-            plugin = self.handler.plugins[id]
+        for id in self.window.app.plugins.plugins:
+            plugin = self.window.app.plugins.plugins[id]
             options = plugin.setup()  # get plugin options
             self.current_plugin = id
 
@@ -151,8 +156,8 @@ class Plugins:
     def save_settings(self):
         """Save plugins settings"""
         selected_plugin = self.current_plugin
-        for id in self.handler.plugins:
-            plugin = self.handler.plugins[id]
+        for id in self.window.app.plugins.plugins:
+            plugin = self.window.app.plugins.plugins[id]
             options = plugin.setup()  # get plugin options
 
             # add plugin to config if not exists
@@ -167,32 +172,32 @@ class Plugins:
                 if option['type'] == 'int' or option['type'] == 'float':
                     if 'slider' in option and option['slider'] \
                             and (option['type'] == 'int' or option['type'] == 'float'):
-                        value = self.window.plugin_option[id][key].slider.value() / option['multiplier']
+                        value = self.window.ui.plugin_option[id][key].slider.value() / option['multiplier']
                     else:
                         if option['type'] == 'int':
                             try:
-                                value = int(self.window.plugin_option[id][key].text())
+                                value = int(self.window.ui.plugin_option[id][key].text())
                             except ValueError:
                                 value = 0
                         elif option['type'] == 'float':
                             try:
-                                value = float(self.window.plugin_option[id][key].text())
+                                value = float(self.window.ui.plugin_option[id][key].text())
                             except ValueError:
                                 value = 0.0
                 elif option['type'] == 'text':
-                    value = self.window.plugin_option[id][key].text()
+                    value = self.window.ui.plugin_option[id][key].text()
                 elif option['type'] == 'textarea':
-                    value = self.window.plugin_option[id][key].toPlainText()
+                    value = self.window.ui.plugin_option[id][key].toPlainText()
                 elif option['type'] == 'bool':
-                    value = self.window.plugin_option[id][key].box.isChecked()
+                    value = self.window.ui.plugin_option[id][key].box.isChecked()
                 elif option['type'] == 'dict':
-                    value = self.window.plugin_option[id][key].model.items
-                self.handler.plugins[id].options[key]['value'] = value
+                    value = self.window.ui.plugin_option[id][key].model.items
+                self.window.app.plugins.plugins[id].options[key]['value'] = value
                 self.window.config.data['plugins'][id][key] = value
 
             # update config if option not exists
             for key in list(self.window.config.data['plugins'].keys()):
-                if key not in self.handler.plugins:
+                if key not in self.window.app.plugins.plugins:
                     self.window.config.data['plugins'].pop(key)
 
         # save config
@@ -219,7 +224,7 @@ class Plugins:
 
         # reload settings window
         self.init_settings()
-        self.window.ui.dialogs.alert(trans('dialog.plugin.settings.defaults.user.result'))
+        # self.window.ui.dialogs.alert(trans('dialog.plugin.settings.defaults.user.result'))
 
     def load_defaults_app(self, force=False):
         """
@@ -233,7 +238,7 @@ class Plugins:
             return
 
         # restore default options
-        self.handler.restore_options(self.current_plugin)
+        self.window.app.plugins.restore_options(self.current_plugin)
 
         # reload settings window
         self.init_settings()
@@ -245,7 +250,7 @@ class Plugins:
 
         :param plugin: Plugin
         """
-        self.handler.register(plugin)
+        self.window.app.plugins.register(plugin)
 
     def unregister(self, id):
         """
@@ -253,8 +258,8 @@ class Plugins:
 
         :param id: Plugin id
         """
-        if self.handler.is_registered(id):
-            self.handler.plugins.pop(id)
+        if self.window.app.plugins.is_registered(id):
+            self.window.app.plugins.plugins.pop(id)
             if id in self.enabled:
                 self.enabled.pop(id)
 
@@ -264,15 +269,15 @@ class Plugins:
 
         :param id: plugin id
         """
-        if self.handler.is_registered(id):
+        if self.window.app.plugins.is_registered(id):
             self.enabled[id] = True
-            self.handler.plugins[id].enabled = True
+            self.window.app.plugins.plugins[id].enabled = True
 
             # dispatch event
             event = Event('enable', {
                 'value': id,
             })
-            self.window.controller.plugins.dispatch(event)
+            self.window.dispatch(event)
 
             self.window.config.data['plugins_enabled'][id] = True
             self.window.config.save()
@@ -291,15 +296,15 @@ class Plugins:
 
         :param id: plugin id
         """
-        if self.handler.is_registered(id):
+        if self.window.app.plugins.is_registered(id):
             self.enabled[id] = False
-            self.handler.plugins[id].enabled = False
+            self.window.app.plugins.plugins[id].enabled = False
 
             # dispatch event
             event = Event('disable', {
                 'value': id,
             })
-            self.window.controller.plugins.dispatch(event)
+            self.window.dispatch(event)
 
             self.window.config.data['plugins_enabled'][id] = False
             self.window.config.save()
@@ -319,7 +324,7 @@ class Plugins:
         :return: true if enabled
         :rtype: bool
         """
-        if self.handler.is_registered(id):
+        if self.window.app.plugins.is_registered(id):
             if id in self.enabled:
                 return self.enabled[id]
         return False
@@ -330,7 +335,7 @@ class Plugins:
 
         :param id: plugin id
         """
-        if self.handler.is_registered(id):
+        if self.window.app.plugins.is_registered(id):
             if self.is_enabled(id):
                 self.disable(id)
             else:
@@ -345,34 +350,49 @@ class Plugins:
         :param idx: tab index
         """
         plugin_idx = 0
-        for id in self.handler.plugins:
-            if self.handler.plugins[id].options:
+        for id in self.window.app.plugins.plugins:
+            if self.window.app.plugins.plugins[id].options:
                 if plugin_idx == idx:
                     self.current_plugin = id
                     break
                 plugin_idx += 1
-        current = self.window.models['plugin.list'].index(idx, 0)
-        self.window.data['plugin.list'].setCurrentIndex(current)
+        current = self.window.ui.models['plugin.list'].index(idx, 0)
+        self.window.ui.nodes['plugin.list'].setCurrentIndex(current)
+
+    def get_plugin_tab_idx(self, plugin_id):
+        """
+        Get plugin tab index
+
+        :param plugin_id: plugin id
+        """
+        plugin_idx = None
+        i = 0
+        for id in self.window.app.plugins.plugins:
+            if id == plugin_id:
+                plugin_idx = i
+                break
+            i += 1
+        return plugin_idx
 
     def update_info(self):
-        """Updates plugins info"""
+        """Update plugins info"""
         enabled_list = []
-        for id in self.handler.plugins:
+        for id in self.window.app.plugins.plugins:
             if self.is_enabled(id):
-                enabled_list.append(self.handler.plugins[id].name)
+                enabled_list.append(self.window.app.plugins.plugins[id].name)
         tooltip = " + ".join(enabled_list)
 
         count_str = ""
         c = 0
-        if len(self.handler.plugins) > 0:
-            for id in self.handler.plugins:
+        if len(self.window.app.plugins.plugins) > 0:
+            for id in self.window.app.plugins.plugins:
                 if self.is_enabled(id):
                     c += 1
 
         if c > 0:
             count_str = "+ " + str(c) + " " + trans('chatbox.plugins')
-        self.window.data['chat.plugins'].setText(count_str)
-        self.window.data['chat.plugins'].setToolTip(tooltip)
+        self.window.ui.nodes['chat.plugins'].setText(count_str)
+        self.window.ui.nodes['chat.plugins'].setToolTip(tooltip)
 
     def load_config(self):
         """
@@ -390,7 +410,7 @@ class Plugins:
         :param value: checkbox option value
         """
         key = id.replace('plugin.' + self.current_plugin + '.', '')
-        self.window.plugin_option[self.current_plugin][key].box.setChecked(value)
+        self.window.ui.plugin_option[self.current_plugin][key].box.setChecked(value)
 
     def config_dict_update(self, id, value):
         """
@@ -401,8 +421,8 @@ class Plugins:
         """
         key = id.replace('plugin.' + self.current_plugin + '.', '')
         values = list(value)
-        self.window.plugin_option[self.current_plugin][key].items = values  # replace model data list
-        self.window.plugin_option[self.current_plugin][key].model.updateData(values)  # update model data
+        self.window.ui.plugin_option[self.current_plugin][key].items = values  # replace model data list
+        self.window.ui.plugin_option[self.current_plugin][key].model.updateData(values)  # update model data
 
     def config_change(self, id, value):
         """
@@ -425,7 +445,7 @@ class Plugins:
                 elif 'max' in option and option['max'] is not None and value > option['max']:
                     value = option['max']
 
-        self.window.plugin_option[self.current_plugin][key].setText('{}'.format(value))
+        self.window.ui.plugin_option[self.current_plugin][key].setText('{}'.format(value))
 
     def config_slider(self, id, value, type=None):
         """
@@ -462,7 +482,7 @@ class Plugins:
             elif 'max' in option and value > option['max']:
                 value = option['max']
 
-            self.window.plugin_option[self.current_plugin][key].input.setText(str(value))
+            self.window.ui.plugin_option[self.current_plugin][key].input.setText(str(value))
 
         slider_value = round(float(value) * multiplier, 0)
         input_value = value
@@ -474,7 +494,7 @@ class Plugins:
         # update from slider
         if type == 'slider':
             txt = '{}'.format(input_value)
-            self.window.plugin_option[self.current_plugin][key].input.setText(txt)
+            self.window.ui.plugin_option[self.current_plugin][key].input.setText(txt)
 
         # update from input
         elif type == 'input':
@@ -482,13 +502,13 @@ class Plugins:
                 slider_value = option['min'] * multiplier
             elif 'max' in option and slider_value > option['max'] * multiplier:
                 slider_value = option['max'] * multiplier
-            self.window.plugin_option[self.current_plugin][key].slider.setValue(slider_value)
+            self.window.ui.plugin_option[self.current_plugin][key].slider.setValue(slider_value)
 
         # update from raw value
         else:
             txt = '{}'.format(value)
-            self.window.plugin_option[self.current_plugin][key].input.setText(txt)
-            self.window.plugin_option[self.current_plugin][key].slider.setValue(slider_value)
+            self.window.ui.plugin_option[self.current_plugin][key].input.setText(txt)
+            self.window.ui.plugin_option[self.current_plugin][key].slider.setValue(slider_value)
 
     def get_option(self, id, key):
         """
@@ -498,7 +518,7 @@ class Plugins:
         :return: option value
         :rtype: any
         """
-        return self.handler.plugins[id].options[key]
+        return self.window.app.plugins.plugins[id].options[key]
 
     def is_type_enabled(self, type):
         """
@@ -508,8 +528,8 @@ class Plugins:
         :rtype: bool
         """
         enabled = False
-        for id in self.handler.plugins:
-            if type in self.handler.plugins[id].type and self.is_enabled(id):
+        for id in self.window.app.plugins.plugins:
+            if type in self.window.app.plugins.plugins[id].type and self.is_enabled(id):
                 enabled = True
                 break
         return enabled
@@ -518,18 +538,18 @@ class Plugins:
         """
         Handle plugin type
         """
-        for type in self.handler.allowed_types:
+        for type in self.window.app.plugins.allowed_types:
             if type == 'audio.input':
                 if self.is_type_enabled(type):
-                    self.window.plugin_addon['audio.input'].setVisible(True)
+                    self.window.ui.plugin_addon['audio.input'].setVisible(True)
                 else:
-                    self.window.plugin_addon['audio.input'].setVisible(False)
+                    self.window.ui.plugin_addon['audio.input'].setVisible(False)
             elif type == 'audio.output':
                 if self.is_type_enabled(type):
                     pass
-                    # self.window.plugin_addon['audio.output'].setVisible(True)
+                    # self.window.ui.plugin_addon['audio.output'].setVisible(True)
                 else:
-                    self.window.plugin_addon['audio.output'].setVisible(False)
+                    self.window.ui.plugin_addon['audio.output'].setVisible(False)
 
     def apply_cmds(self, ctx, cmds):
         """
@@ -546,22 +566,9 @@ class Plugins:
         if len(commands) == 0:
             return
 
-        # dispatch 'command' event
+        # dispatch 'cmd.execute' event
         event = Event('cmd.execute', {
             'commands': commands
         })
         event.ctx = ctx
-        self.dispatch(event)
-
-    def dispatch(self, event, all=False):
-        """
-        Dispatch event to plugins
-
-        :param event: event to dispatch
-        :param all: true if dispatch to all plugins (enabled or not)
-        """
-        for id in self.handler.plugins:
-            if self.is_enabled(id) or all:
-                if event.stop:
-                    break
-                self.window.dispatcher.dispatch(id, event)
+        self.window.controller.command.dispatch(event)
