@@ -6,11 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2023.12.17 22:00:00                  #
+# Updated Date: 2023.12.20 22:00:00                  #
 # ================================================== #
 import os
+import copy
 
-from showinfm import show_in_file_manager
 from ..utils import trans
 
 
@@ -47,7 +47,7 @@ class Settings:
         self.load_config_options()
 
         # store copy of loaded config data
-        self.before_config = dict(self.window.config.all())
+        self.before_config = copy.deepcopy(self.window.config.all())
 
     def load_config_options(self, initialize=True):
         """
@@ -55,7 +55,7 @@ class Settings:
 
         :param initialize: True if marks settings as initialized
         """
-        self.options = self.window.settings.get_options()
+        self.options = self.window.app.settings.get_options()
         if initialize:
             self.initialized = True
 
@@ -72,14 +72,13 @@ class Settings:
                     continue
                 type = self.options[option]['type']
                 if type == 'text':
-                    self.window.config.set(option, self.window.config_option[option].text())
+                    self.window.config.set(option, self.window.ui.config_option[option].text())
                 elif type == 'textarea':
-                    self.window.config.set(option, self.window.config_option[option].toPlainText())
+                    self.window.config.set(option, self.window.ui.config_option[option].toPlainText())
 
         info = trans('info.settings.saved')
         self.window.config.save()
         self.window.set_status(info)
-        self.close_window(id)
         self.update_font_size()
         self.window.controller.ui.update()
 
@@ -87,6 +86,9 @@ class Settings:
         if self.before_config['layout.density'] != self.window.config.get('layout.density'):
             self.window.controller.theme.reload()
 
+        self.before_config = copy.deepcopy(self.window.config.all())
+        self.close_window(id)
+
     def save_all(self):
         """Save all settings"""
         info = trans('info.settings.all.saved')
@@ -104,14 +106,7 @@ class Settings:
 
     def update_font_size(self):
         """Update font size"""
-        self.window.data['output'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
-        self.window.data['input'].setStyleSheet(self.window.controller.theme.get_style('chat_input'))
-        self.window.data['ctx.contexts'].setStyleSheet(self.window.controller.theme.get_style('ctx.contexts'))
-        self.window.data['notepad1'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
-        self.window.data['notepad2'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
-        self.window.data['notepad3'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
-        self.window.data['notepad4'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
-        self.window.data['notepad5'].setStyleSheet(self.window.controller.theme.get_style('chat_output'))
+        self.window.controller.theme.apply(False)
 
     def toggle_settings(self, id):
         """
@@ -119,16 +114,16 @@ class Settings:
 
         :param id: settings id
         """
-        if id in self.window.settings.active and self.window.settings.active[id]:
+        if id in self.window.app.settings.active and self.window.app.settings.active[id]:
             self.close_window(id)
         else:
             self.window.ui.dialogs.open('config.' + id, width=self.width, height=self.height)
             self.init(id)
-            self.window.settings.active[id] = True
+            self.window.app.settings.active[id] = True
 
             # if no API key, focus on API key input
             if self.window.config.get('api_key') is None or self.window.config.get('api_key') == '':
-                self.window.config_option['api_key'].setFocus()
+                self.window.ui.config_option['api_key'].setFocus()
 
         # update menu
         self.update()
@@ -140,18 +135,18 @@ class Settings:
         :param file: JSON file to load
         """
         id = 'editor'
-        current_file = self.window.dialog['config.editor'].file
-        if id in self.window.settings.active and self.window.settings.active[id]:
+        current_file = self.window.ui.dialog['config.editor'].file
+        if id in self.window.app.settings.active and self.window.app.settings.active[id]:
             if current_file == file:
                 self.window.ui.dialogs.close('config.' + id)
-                self.window.settings.active[id] = False
+                self.window.app.settings.active[id] = False
             else:
-                self.window.settings.load_editor(file)  # load file to editor
-                self.window.dialog['config.editor'].file = file
+                self.window.app.settings.load_editor(file)  # load file to editor
+                self.window.ui.dialog['config.editor'].file = file
         else:
-            self.window.settings.load_editor(file)  # load file to editor
+            self.window.app.settings.load_editor(file)  # load file to editor
             self.window.ui.dialogs.open('config.' + id, width=self.width, height=self.height)
-            self.window.settings.active[id] = True
+            self.window.app.settings.active[id] = True
 
         # update menu
         self.update()
@@ -162,9 +157,9 @@ class Settings:
 
         :param id: settings window id
         """
-        if id in self.window.settings.active and self.window.settings.active[id]:
+        if id in self.window.app.settings.active and self.window.app.settings.active[id]:
             self.window.ui.dialogs.close('config.' + id)
-            self.window.settings.active[id] = False
+            self.window.app.settings.active[id] = False
 
     def close(self, id):
         """
@@ -172,12 +167,12 @@ class Settings:
 
         :param id: settings window id
         """
-        if id in self.window.menu:
-            self.window.menu[id].setChecked(False)
+        if id in self.window.ui.menu:
+            self.window.ui.menu[id].setChecked(False)
 
         allowed_settings = ['settings']
-        if id in allowed_settings and id in self.window.menu:
-            self.window.menu[id].setChecked(False)
+        if id in allowed_settings and id in self.window.ui.menu:
+            self.window.ui.menu[id].setChecked(False)
 
     def update(self):
         """Update settings"""
@@ -185,13 +180,13 @@ class Settings:
 
     def update_menu(self):
         """Update menu"""
-        for id in self.window.settings.ids:
+        for id in self.window.app.settings.ids:
             key = 'config.' + id
-            if key in self.window.menu:
-                if id in self.window.settings.active and self.window.settings.active[id]:
-                    self.window.menu['config.' + id].setChecked(True)
+            if key in self.window.ui.menu:
+                if id in self.window.app.settings.active and self.window.app.settings.active[id]:
+                    self.window.ui.menu['config.' + id].setChecked(True)
                 else:
-                    self.window.menu['config.' + id].setChecked(False)
+                    self.window.ui.menu['config.' + id].setChecked(False)
 
     def init(self, id):
         """
@@ -239,13 +234,13 @@ class Settings:
 
                 # call vision checkboxes events
                 if id == "vision.capture.enabled":
-                    self.window.data['vision.capture.enable'].setChecked(value)
+                    self.window.ui.nodes['vision.capture.enable'].setChecked(value)
                 if id == "vision.capture.auto":
-                    self.window.data['vision.capture.auto'].setChecked(value)
+                    self.window.ui.nodes['vision.capture.auto'].setChecked(value)
 
         # update checkbox
-        if id in self.window.config_option and value is not None:
-            self.window.config_option[id].box.setChecked(value)
+        if id in self.window.ui.config_option and value is not None:
+            self.window.ui.config_option[id].box.setChecked(value)
 
     def change(self, id, value, section=None):
         """
@@ -255,7 +250,6 @@ class Settings:
         :param value: input option value
         :param section: settings section
         """
-        self.before_config = dict(self.window.config.all())
 
         # dialog: preset
         if id.startswith('preset.'):
@@ -294,7 +288,7 @@ class Settings:
             self.update_font_size()
 
         txt = '{}'.format(value)
-        self.window.config_option[id].setText(txt)
+        self.window.ui.config_option[id].setText(txt)
 
     def apply(self, id, value, type=None, section=None):
         """
@@ -305,7 +299,6 @@ class Settings:
         :param type: option type (slider, input, None)
         :param section: option section (settings, preset.editor, None)
         """
-        self.before_config = dict(self.window.config.all())
 
         # dialog: preset
         if id.startswith('preset.'):
@@ -317,8 +310,6 @@ class Settings:
             self.window.controller.plugins.config_slider(id, value, type)
             return
 
-        orig_value = value
-
         # dialog: settings or global settings
         option_type = None
         multiplier = 1  # default multiplier
@@ -334,7 +325,7 @@ class Settings:
                         value = int(value)
                     except:
                         value = 0
-                        self.window.config_option[id].input.setText(str(value))
+                        self.window.ui.config_option[id].input.setText(str(value))
                 elif option_type == 'float':
                     if 'multiplier' in option:
                         multiplier = option['multiplier']
@@ -343,12 +334,12 @@ class Settings:
                                 value = float(value)
                             except:
                                 value = option['min']
-                                self.window.config_option[id].input.setText(str(value))
+                                self.window.ui.config_option[id].input.setText(str(value))
                             if value < option['min']:
                                 value = option['min']
                             elif value > option['max']:
                                 value = option['max']
-                            self.window.config_option[id].input.setText(str(value))
+                            self.window.ui.config_option[id].input.setText(str(value))
 
         if id in self.float_values:
             multiplier = self.float_values[id]['multiplier']
@@ -372,13 +363,13 @@ class Settings:
                         value = int(value)
                 except:
                     value = min
-                    self.window.config_option[id].input.setText(str(value))
+                    self.window.ui.config_option[id].input.setText(str(value))
                 # fix min max values
                 if value < min:
                     value = min
                 elif value > max:
                     value = max
-                self.window.config_option[id].input.setText(str(value))
+                self.window.ui.config_option[id].input.setText(str(value))
 
         # prepare slider value
         slider_value = round(float(value) * multiplier, 0)
@@ -395,7 +386,7 @@ class Settings:
             preset = self.window.config.get('preset')  # current preset
             is_current = True
             if section == 'preset.editor':
-                preset = self.window.config_option['preset.filename'].text()  # editing preset
+                preset = self.window.ui.config_option['preset.filename'].text()  # editing preset
                 is_current = False
             self.window.controller.presets.update_field(id, input_value, preset, is_current)
             self.window.controller.presets.update_field('preset.temperature', input_value, True)
@@ -410,7 +401,7 @@ class Settings:
         # update from slider
         if type == 'slider':
             txt = '{}'.format(input_value)
-            self.window.config_option[id].input.setText(txt)
+            self.window.ui.config_option[id].input.setText(txt)
 
         # update from input
         elif type == 'input':
@@ -433,10 +424,10 @@ class Settings:
                 elif slider_value > max:
                     slider_value = max
             if id in self.options and self.options[id]['slider']:
-                self.window.config_option[id].slider.setValue(slider_value)
+                self.window.ui.config_option[id].slider.setValue(slider_value)
         else:
             if id in self.options and self.options[id]['slider']:
-                self.window.config_option[id].slider.setValue(slider_value)
+                self.window.ui.config_option[id].slider.setValue(slider_value)
 
         # update from raw value
         if id.startswith('font_size'):
@@ -454,15 +445,15 @@ class Settings:
         :param value: value
         :param section: section
         """
-        if id not in self.window.groups:
+        if id not in self.window.ui.groups:
             return
 
-        self.window.groups[id].collapse(value)
+        self.window.ui.groups[id].collapse(value)
 
     def open_config_dir(self):
         """Open user config directory"""
         if os.path.exists(self.window.config.path):
-            show_in_file_manager(self.window.config.path)
+            self.window.controller.files.open_in_file_manager(self.window.config.path, True)
         else:
             self.window.set_status('Config directory not exists: {}'.format(self.window.config.path))
 
@@ -477,11 +468,11 @@ class Settings:
             return
 
         # load default user config
-        self.window.settings.load_user_settings()
+        self.window.app.settings.load_user_settings()
 
         # re-init settings
         self.window.controller.settings.init('settings')
-        self.window.ui.dialogs.alert(trans('dialog.settings.defaults.user.result'))
+        # self.window.ui.dialogs.alert(trans('dialog.settings.defaults.user.result'))
 
     def load_defaults_app(self, force=False):
         """
@@ -494,7 +485,7 @@ class Settings:
             return
 
         # load default user config
-        self.window.settings.load_app_settings()
+        self.window.app.settings.load_app_settings()
 
         # re-init settings
         self.window.controller.settings.init('settings')
