@@ -6,13 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2023.12.18 04:00:00                  #
+# Updated Date: 2023.12.20 18:00:00                  #
 # ================================================== #
 import os
 import threading
 import time
 
-from PySide6.QtCore import QObject, Signal, Qt, Slot
+from PySide6.QtCore import QObject, Signal, Slot
 import speech_recognition as sr
 from openai import OpenAI
 import audioop
@@ -35,10 +35,12 @@ class Plugin(BasePlugin):
         self.thread = None
         self.listening = False
         self.waiting = False
+        self.stop = False
         self.magic_word_detected = False
         self.is_first_adjust = True
         self.empty_phrases = ['Thank you for watching']  # phrases to ignore (fix for empty phrases)
         self.order = 1
+        self.use_locale = True
         self.init_options()
 
     def init_options(self):
@@ -50,14 +52,16 @@ class Plugin(BasePlugin):
                         "Specify model, default: whisper-1")
         self.add_option("timeout", "int", 2,
                         "Timeout",
-                        "Speech recognition timeout. Default: 2", min=0, max=30, slider=True, tooltip="Timeout, default: 2")
+                        "Speech recognition timeout. Default: 2", min=0, max=30, slider=True,
+                        tooltip="Timeout, default: 2")
         self.add_option("phrase_length", "int", 4,
                         "Phrase max length",
-                        "Speech recognition phrase length. Default: 4", min=0, max=30, slider=True, tooltip="Phrase max length, "
-                                                                                                "default: 4")
+                        "Speech recognition phrase length. Default: 4", min=0, max=30, slider=True,
+                        tooltip="Phrase max length, default: 4")
         self.add_option("min_energy", "float", 1.3,
                         "Min. energy",
-                        "Minimum threshold multiplier above the noise level to begin recording; 1 = disabled. Default: 1.3",
+                        "Minimum threshold multiplier above the noise level to begin recording; 1 = disabled. "
+                        "Default: 1.3",
                         min=1, max=50, slider=True,
                         tooltip="Min. energy, default: 1.3, 1 = disabled, adjust for your microphone", multiplier=10)
         self.add_option("adjust_noise", "bool", True,
@@ -89,10 +93,12 @@ class Plugin(BasePlugin):
                         "Magic word option must be enabled, examples: \"Hey GPT, OK GPT\"")
         self.add_option("magic_word_timeout", "int", 1,
                         "Magic word timeout",
-                        "Magic word recognition timeout. Default: 1", min=0, max=30, slider=True, tooltip="Timeout, default: 1")
+                        "Magic word recognition timeout. Default: 1", min=0, max=30, slider=True,
+                        tooltip="Timeout, default: 1")
         self.add_option("magic_word_phrase_length", "int", 2,
                         "Magic word phrase max length",
-                        "Magic word phrase length. Default: 2", min=0, max=30, slider=True, tooltip="Phrase length, default: 2")
+                        "Magic word phrase length. Default: 2", min=0, max=30, slider=True,
+                        tooltip="Phrase length, default: 2")
         self.add_option("prefix_words", "text", "",
                         "Prefix words",
                         "Specify prefix words: if defined, only phrases starting with these words will be transmitted, "
@@ -108,7 +114,8 @@ class Plugin(BasePlugin):
         # advanced options
         self.add_option("recognition_energy_threshold", "int", 300,
                         "energy_threshold",
-                        "Represents the energy level threshold for sounds. Default: 300", min=0, max=10000, slider=True, advanced=True)
+                        "Represents the energy level threshold for sounds. Default: 300", min=0, max=10000,
+                        slider=True, advanced=True)
         self.add_option("recognition_dynamic_energy_threshold", "bool", True,
                         "dynamic_energy_threshold",
                         "Represents whether the energy level threshold "
@@ -196,11 +203,12 @@ class Plugin(BasePlugin):
         :param state: state to set
         """
         self.speech_enabled = state
-        self.window.plugin_addon['audio.input'].btn_toggle.setChecked(state)
+        self.window.ui.plugin_addon['audio.input'].btn_toggle.setChecked(state)
 
         # Start thread if not started
         if state:
             self.listening = True
+            self.stop = False
             self.handle_thread()
         else:
             self.listening = False
@@ -238,6 +246,8 @@ class Plugin(BasePlugin):
                 self.on_disable()
         elif name == 'audio.input.toggle':
             self.toggle_speech(data['value'])
+        elif name == 'audio.input.stop':
+            self.on_stop()
 
     def on_ctx_begin(self, ctx):
         """
@@ -262,7 +272,18 @@ class Plugin(BasePlugin):
 
     def on_disable(self):
         """Event: On plugin disable"""
-        self.speech_enabled = True
+        self.speech_enabled = False
+        self.listening = False
+        self.stop = True
+        self.window.ui.plugin_addon['audio.input'].btn_toggle.setChecked(False)
+        self.set_status('')
+
+    def on_stop(self):
+        """Event: On input stop"""
+        self.stop = True
+        self.listening = False
+        self.speech_enabled = False
+        self.set_status('')
 
     def on_input_before(self, text):
         """
@@ -279,6 +300,7 @@ class Plugin(BasePlugin):
         self.waiting = True
         self.listening = False
         self.thread_started = False
+        self.set_status('')
 
     def handle_thread(self):
         """
@@ -317,7 +339,7 @@ class Plugin(BasePlugin):
 
         :param status: status
         """
-        self.window.plugin_addon['audio.input'].set_status(status)
+        self.window.ui.plugin_addon['audio.input'].set_status(status)
 
     @Slot(str)
     def handle_input(self, text):
@@ -371,13 +393,13 @@ class Plugin(BasePlugin):
             if not magic_prev_detected:
                 if not is_magic_word:
                     self.window.set_status(trans('audio.magic_word.invalid'))
-                self.window.data['input'].setText(text)
+                self.window.ui.nodes['input'].setText(text)
                 return
             else:
                 self.window.set_status("")
 
         # update input text
-        self.window.data['input'].setText(text)
+        self.window.ui.nodes['input'].setText(text)
 
         # send text
         if self.get_option_value('auto_send'):
@@ -392,13 +414,15 @@ class Plugin(BasePlugin):
         Insert text to input and send
         """
         self.thread_started = False
+        self.set_status('')
 
     @Slot()
     def handle_started(self):
         """
         Handle listening started
         """
-        print("Whisper is listening...")
+        pass
+        # print("Whisper is listening...")
 
     @Slot()
     def handle_stop(self):
@@ -408,7 +432,8 @@ class Plugin(BasePlugin):
         self.thread_started = False
         self.listening = False
         self.window.set_status("")
-        print("Whisper stopped listening...")
+        self.set_status('')
+        # print("Whisper stopped listening...")
         self.toggle_speech(False)
 
 
@@ -427,10 +452,14 @@ class AudioInputThread(QObject):
 
     def run(self):
         try:
+            if not self.plugin.listening:
+                return
+
             client = OpenAI(
                 api_key=self.plugin.window.config.get('api_key'),
                 organization=self.plugin.window.config.get('organization_key'),
             )
+
             print("Starting audio listener....")
             path = os.path.join(self.plugin.window.config.path, 'input.wav')
 
@@ -441,6 +470,13 @@ class AudioInputThread(QObject):
                 while self.plugin.listening and not self.plugin.window.is_closing:
                     self.plugin.set_status('')
 
+                    if self.plugin.stop:
+                        self.plugin.stop = False
+                        self.plugin.listening = False
+                        self.plugin.set_status('Stop.')
+                        self.stopped.emit()
+                        break
+
                     if not self.plugin.can_listen():
                         time.sleep(0.5)
                         continue
@@ -504,8 +540,9 @@ class AudioInputThread(QObject):
                             # check RMS / energy
                             rms = audioop.rms(raw_data, 2)
                             if min_energy > 0:
-                                self.plugin.window.set_status("{}: {} / {} (x{})".format(trans('audio.speak.energy'),
-                                                                                           rms, int(ambient_noise_energy), min_energy))
+                                self.plugin.window.set_status("{}: {} / {} (x{})".
+                                                              format(trans('audio.speak.energy'),
+                                                                     rms, int(ambient_noise_energy), min_energy))
                             if rms < ambient_noise_energy:
                                 continue
 
@@ -532,7 +569,7 @@ class AudioInputThread(QObject):
                                             is_empty_phrase = True
                                             break
 
-                                    if is_empty_phrase or transcript.strip() == '':
+                                    if is_empty_phrase:
                                         continue
 
                                     if self.plugin.can_listen():
@@ -550,10 +587,10 @@ class AudioInputThread(QObject):
                             self.plugin.set_status('')  # clear status
                             break
                     except Exception as e:
-                        print(f"An error occurred: {str(e)}\n")  # timeout
+                        print("Speech recognition error: {}".format(str(e)))
 
             self.destroyed.emit()
 
         except Exception as e:
             self.destroyed.emit()
-            print(e)
+            print("Audio input thread error: {}".format(str(e)))
