@@ -43,6 +43,7 @@ class Plugin(BasePlugin):
             "file_size",
             "file_info",
         ]
+        self.use_locale = True
         self.init_options()
 
     def init_options(self):
@@ -163,7 +164,7 @@ class Plugin(BasePlugin):
         ctx = event.ctx
 
         if name == 'cmd.syntax':
-            data['value'] = self.cmd_syntax(data['value'])
+            self.cmd_syntax(data)
         elif name == 'cmd.execute':
             self.cmd(ctx, data['commands'])
 
@@ -189,20 +190,17 @@ class Plugin(BasePlugin):
         self.window.log('[CMD] ' + str(msg))
         print('[CMD] ' + str(msg))
 
-    def cmd_syntax(self, syntax):
+    def cmd_syntax(self, data):
         """
         Event: On cmd syntax prepare
 
-        :param syntax: syntax
-        :return: syntax
-        :rtype: str
+        :param data: event data dict
         """
         for option in self.allowed_cmds:
             if self.is_cmd_allowed(option):
                 key = "syntax_" + option
                 if self.has_option(key):
-                    syntax += "\n" + str(self.get_option_value(key))
-        return syntax
+                    data['syntax'].append(str(self.get_option_value(key)))
 
     def cmd(self, ctx, cmds):
         """
